Exception: MCP::Client::HTTP::InsecureURLError

Inherits:
ArgumentError
  • Object
show all
Defined in:
lib/mcp/client/http.rb

Overview

Raised when an ‘oauth:` provider is paired with an MCP URL that is neither HTTPS nor a loopback `http://` URL, since a bearer token sent over plain HTTP to a remote host is trivially observed and stolen.