Exception: MCP::Client::OAuth::Provider::InvalidClientIDMetadataDocumentURLError

Inherits:
ArgumentError
  • Object
show all
Defined in:
lib/mcp/client/oauth/provider.rb

Overview

Raised when ‘client_id_metadata_document_url` is provided but does not meet the structural requirements for a Client ID Metadata Document URL: HTTPS, non-root path, and no fragment, query, userinfo, or `.`/`..` segments. The CIMD URL is sent to the authorization server as the OAuth `client_id`, so the same Communication Security guarantee that protects the redirect URI applies and the value must unambiguously identify the document.