Class: MCP::Server

Inherits:
Object
  • Object
show all
Includes:
Instrumentation, Pagination
Defined in:
lib/mcp/server.rb,
lib/mcp/server/pagination.rb,
lib/mcp/server/transports.rb,
lib/mcp/server/capabilities.rb,
lib/mcp/server/transports/stdio_transport.rb,
lib/mcp/server/transports/streamable_http_transport.rb

Defined Under Namespace

Modules: Pagination, Transports Classes: Capabilities, MethodAlreadyDefinedError, RequestHandlerError, ResourceNotFoundError, URLElicitationRequiredError, ValidationError

Constant Summary collapse

DEFAULT_VERSION =
"0.1.0"
UNSUPPORTED_PROPERTIES_UNTIL_2025_06_18 =
[:description, :icons].freeze
UNSUPPORTED_PROPERTIES_UNTIL_2025_03_26 =
[:title, :websiteUrl].freeze
DEFAULT_COMPLETION_RESULT =
{ completion: { values: [], hasMore: false } }.freeze
MAX_COMPLETION_VALUES =

Servers return an array of completion values ranked by relevance, with maximum 100 items per response. modelcontextprotocol.io/specification/2025-11-25/server/utilities/completion#completion-results

100

Instance Attribute Summary collapse

Instance Method Summary collapse

Methods included from Instrumentation

#add_instrumentation_data, #instrument_call

Constructor Details

#initialize(description: nil, icons: [], name: "model_context_protocol", title: nil, version: DEFAULT_VERSION, website_url: nil, instructions: nil, tools: [], prompts: [], resources: [], resource_templates: [], server_context: nil, configuration: nil, capabilities: nil, page_size: nil, transport: nil) ⇒ Server

Returns a new instance of Server.



108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
135
136
137
138
139
140
141
142
143
144
145
146
147
148
149
150
151
152
153
154
155
156
157
158
159
160
161
162
163
164
165
166
167
168
169
170
171
172
173
174
# File 'lib/mcp/server.rb', line 108

def initialize(
  description: nil,
  icons: [],
  name: "model_context_protocol",
  title: nil,
  version: DEFAULT_VERSION,
  website_url: nil,
  instructions: nil,
  tools: [],
  prompts: [],
  resources: [],
  resource_templates: [],
  server_context: nil,
  configuration: nil,
  capabilities: nil,
  page_size: nil,
  transport: nil
)
  @description = description
  @icons = icons
  @name = name
  @title = title
  @version = version
  @website_url = website_url
  @instructions = instructions
  @tool_names = tools.map(&:name_value)
  @tools = tools.to_h { |t| [t.name_value, t] }
  @prompts = prompts.to_h { |p| [p.name_value, p] }
  @resources = resources
  @resource_templates = resource_templates
  @resource_index = index_resources_by_uri(resources)
  @server_context = server_context
  self.page_size = page_size
  @configuration = MCP.configuration.merge(configuration)
  @client = nil

  validate!

  # Accept either a plain Hash or an `MCP::Server::Capabilities` builder.
  @capabilities = if capabilities.is_a?(Capabilities)
    capabilities.to_h
  else
    capabilities || default_capabilities
  end
  @client_capabilities = nil
  @logging_message_notification = nil

  @handlers = {
    Methods::RESOURCES_LIST => method(:list_resources),
    Methods::RESOURCES_READ => method(:read_resource_no_content),
    Methods::RESOURCES_TEMPLATES_LIST => method(:list_resource_templates),
    Methods::RESOURCES_SUBSCRIBE => ->(_) { {} },
    Methods::RESOURCES_UNSUBSCRIBE => ->(_) { {} },
    Methods::TOOLS_LIST => method(:list_tools),
    Methods::TOOLS_CALL => method(:call_tool),
    Methods::PROMPTS_LIST => method(:list_prompts),
    Methods::PROMPTS_GET => method(:get_prompt),
    Methods::INITIALIZE => method(:init),
    Methods::PING => ->(_) { {} },
    Methods::NOTIFICATIONS_INITIALIZED => ->(_) {},
    Methods::NOTIFICATIONS_PROGRESS => ->(_) {},
    Methods::NOTIFICATIONS_ROOTS_LIST_CHANGED => ->(_) {},
    Methods::COMPLETION_COMPLETE => ->(_) { DEFAULT_COMPLETION_RESULT },
    Methods::LOGGING_SET_LEVEL => method(:configure_logging_level),
  }
  @transport = transport
end

Instance Attribute Details

#capabilitiesObject

Returns the value of attribute capabilities.



105
106
107
# File 'lib/mcp/server.rb', line 105

def capabilities
  @capabilities
end

#client_capabilitiesObject (readonly)

Returns the value of attribute client_capabilities.



106
107
108
# File 'lib/mcp/server.rb', line 106

def client_capabilities
  @client_capabilities
end

#configurationObject

Returns the value of attribute configuration.



105
106
107
# File 'lib/mcp/server.rb', line 105

def configuration
  @configuration
end

#descriptionObject

Returns the value of attribute description.



105
106
107
# File 'lib/mcp/server.rb', line 105

def description
  @description
end

#iconsObject

Returns the value of attribute icons.



105
106
107
# File 'lib/mcp/server.rb', line 105

def icons
  @icons
end

#instructionsObject

Returns the value of attribute instructions.



105
106
107
# File 'lib/mcp/server.rb', line 105

def instructions
  @instructions
end

#logging_message_notificationObject

Returns the value of attribute logging_message_notification.



105
106
107
# File 'lib/mcp/server.rb', line 105

def logging_message_notification
  @logging_message_notification
end

#nameObject

Returns the value of attribute name.



105
106
107
# File 'lib/mcp/server.rb', line 105

def name
  @name
end

#page_sizeObject

Returns the value of attribute page_size.



106
107
108
# File 'lib/mcp/server.rb', line 106

def page_size
  @page_size
end

#promptsObject

Returns the value of attribute prompts.



105
106
107
# File 'lib/mcp/server.rb', line 105

def prompts
  @prompts
end

#resourcesObject

Returns the value of attribute resources.



105
106
107
# File 'lib/mcp/server.rb', line 105

def resources
  @resources
end

#server_contextObject

Returns the value of attribute server_context.



105
106
107
# File 'lib/mcp/server.rb', line 105

def server_context
  @server_context
end

#titleObject

Returns the value of attribute title.



105
106
107
# File 'lib/mcp/server.rb', line 105

def title
  @title
end

#toolsObject

Returns the value of attribute tools.



105
106
107
# File 'lib/mcp/server.rb', line 105

def tools
  @tools
end

#transportObject

Returns the value of attribute transport.



105
106
107
# File 'lib/mcp/server.rb', line 105

def transport
  @transport
end

#versionObject

Returns the value of attribute version.



105
106
107
# File 'lib/mcp/server.rb', line 105

def version
  @version
end

#website_urlObject

Returns the value of attribute website_url.



105
106
107
# File 'lib/mcp/server.rb', line 105

def website_url
  @website_url
end

Instance Method Details

#build_sampling_params(capabilities, messages:, max_tokens:, system_prompt: nil, model_preferences: nil, include_context: nil, temperature: nil, stop_sequences: nil, metadata: nil, tools: nil, tool_choice: nil) ⇒ Object



316
317
318
319
320
321
322
323
324
325
326
327
328
329
330
331
332
333
334
335
336
337
338
339
340
341
342
343
344
345
346
347
348
349
350
351
352
353
# File 'lib/mcp/server.rb', line 316

def build_sampling_params(
  capabilities,
  messages:,
  max_tokens:,
  system_prompt: nil,
  model_preferences: nil,
  include_context: nil,
  temperature: nil,
  stop_sequences: nil,
  metadata: nil,
  tools: nil,
  tool_choice: nil
)
  unless capabilities&.dig(:sampling)
    raise "Client does not support sampling."
  end

  if tools && !capabilities.dig(:sampling, :tools)
    raise "Client does not support sampling with tools."
  end

  if tool_choice && !capabilities.dig(:sampling, :tools)
    raise "Client does not support sampling with tool_choice."
  end

  {
    messages: messages,
    maxTokens: max_tokens,
    systemPrompt: system_prompt,
    modelPreferences: model_preferences,
    includeContext: include_context,
    temperature: temperature,
    stopSequences: stop_sequences,
    metadata: ,
    tools: tools,
    toolChoice: tool_choice,
  }.compact
end

#completion_handler {|params| ... } ⇒ Object

Sets a custom handler for ‘completion/complete` requests. The block receives the parsed request params and should return completion values.

Yields:

  • (params)

    The request params containing ‘:ref`, `:argument`, and optionally `:context`.

Yield Returns:

  • (Hash)

    A hash with ‘:completion` key containing `:values`, optional `:total`, and `:hasMore`.



294
295
296
# File 'lib/mcp/server.rb', line 294

def completion_handler(&block)
  @handlers[Methods::COMPLETION_COMPLETE] = block
end

#define_custom_method(method_name:, &block) ⇒ Object



219
220
221
222
223
224
225
# File 'lib/mcp/server.rb', line 219

def define_custom_method(method_name:, &block)
  if @handlers.key?(method_name)
    raise MethodAlreadyDefinedError, method_name
  end

  @handlers[method_name] = block
end

#define_prompt(name: nil, title: nil, description: nil, arguments: [], &block) ⇒ Object



212
213
214
215
216
217
# File 'lib/mcp/server.rb', line 212

def define_prompt(name: nil, title: nil, description: nil, arguments: [], &block)
  prompt = Prompt.define(name: name, title: title, description: description, arguments: arguments, &block)
  @prompts[prompt.name_value] = prompt

  validate!
end

#define_tool(name: nil, title: nil, description: nil, input_schema: nil, output_schema: nil, annotations: nil, meta: nil, &block) ⇒ Object



202
203
204
205
206
207
208
209
210
# File 'lib/mcp/server.rb', line 202

def define_tool(name: nil, title: nil, description: nil, input_schema: nil, output_schema: nil, annotations: nil, meta: nil, &block)
  tool = Tool.define(name: name, title: title, description: description, input_schema: input_schema, output_schema: output_schema, annotations: annotations, meta: meta, &block)
  tool_name = tool.name_value

  @tool_names << tool_name
  @tools[tool_name] = tool

  validate!
end

#handle(request, session: nil) ⇒ Hash?

Processes a parsed JSON-RPC request and returns the response as a Hash.

Parameters:

  • request (Hash)

    A parsed JSON-RPC request.

  • session (ServerSession, nil) (defaults to: nil)

    Per-connection session. Passed by ‘ServerSession#handle` for session-scoped notification delivery. When `nil`, progress and logging notifications from tool handlers are silently skipped.

Returns:

  • (Hash, nil)

    The JSON-RPC response, or ‘nil` for notifications.



183
184
185
186
187
# File 'lib/mcp/server.rb', line 183

def handle(request, session: nil)
  JsonRpcHandler.handle(request) do |method, request_id|
    handle_request(request, method, session: session, related_request_id: request_id)
  end
end

#handle_json(request, session: nil) ⇒ String?

Processes a JSON-RPC request string and returns the response as a JSON string.

Parameters:

  • request (String)

    A JSON-RPC request as a JSON string.

  • session (ServerSession, nil) (defaults to: nil)

    Per-connection session. Passed by ‘ServerSession#handle_json` for session-scoped notification delivery. When `nil`, progress and logging notifications from tool handlers are silently skipped.

Returns:

  • (String, nil)

    The JSON-RPC response as JSON, or ‘nil` for notifications.



196
197
198
199
200
# File 'lib/mcp/server.rb', line 196

def handle_json(request, session: nil)
  JsonRpcHandler.handle_json(request) do |method, request_id|
    handle_request(request, method, session: session, related_request_id: request_id)
  end
end

#notify_log_message(data:, level:, logger: nil) ⇒ Object



259
260
261
262
263
264
265
266
267
268
269
# File 'lib/mcp/server.rb', line 259

def notify_log_message(data:, level:, logger: nil)
  return unless @transport
  return unless logging_message_notification&.should_notify?(level)

  params = { "data" => data, "level" => level }
  params["logger"] = logger if logger

  @transport.send_notification(Methods::NOTIFICATIONS_MESSAGE, params)
rescue => e
  report_exception(e, { notification: "log_message" })
end

#notify_prompts_list_changedObject



243
244
245
246
247
248
249
# File 'lib/mcp/server.rb', line 243

def notify_prompts_list_changed
  return unless @transport

  @transport.send_notification(Methods::NOTIFICATIONS_PROMPTS_LIST_CHANGED)
rescue => e
  report_exception(e, { notification: "prompts_list_changed" })
end

#notify_resources_list_changedObject



251
252
253
254
255
256
257
# File 'lib/mcp/server.rb', line 251

def notify_resources_list_changed
  return unless @transport

  @transport.send_notification(Methods::NOTIFICATIONS_RESOURCES_LIST_CHANGED)
rescue => e
  report_exception(e, { notification: "resources_list_changed" })
end

#notify_tools_list_changedObject



235
236
237
238
239
240
241
# File 'lib/mcp/server.rb', line 235

def notify_tools_list_changed
  return unless @transport

  @transport.send_notification(Methods::NOTIFICATIONS_TOOLS_LIST_CHANGED)
rescue => e
  report_exception(e, { notification: "tools_list_changed" })
end

#resources_read_handler {|params| ... } ⇒ Object

Sets a custom handler for ‘resources/read` requests. The block receives the parsed request params and should return resource contents. The return value is set as the `contents` field of the response.

Yields:

  • (params)

    The request params containing ‘:uri`.

Yield Returns:

  • (Array<Hash>, Hash)

    Resource contents.



285
286
287
# File 'lib/mcp/server.rb', line 285

def resources_read_handler(&block)
  @handlers[Methods::RESOURCES_READ] = block
end

#resources_subscribe_handler {|params| ... } ⇒ Object

Sets a custom handler for ‘resources/subscribe` requests. The block receives the parsed request params. The return value is ignored; the response is always an empty result `{}` per the MCP specification.

Yields:

  • (params)

    The request params containing ‘:uri`.



303
304
305
# File 'lib/mcp/server.rb', line 303

def resources_subscribe_handler(&block)
  @handlers[Methods::RESOURCES_SUBSCRIBE] = block
end

#resources_unsubscribe_handler {|params| ... } ⇒ Object

Sets a custom handler for ‘resources/unsubscribe` requests. The block receives the parsed request params. The return value is ignored; the response is always an empty result `{}` per the MCP specification.

Yields:

  • (params)

    The request params containing ‘:uri`.



312
313
314
# File 'lib/mcp/server.rb', line 312

def resources_unsubscribe_handler(&block)
  @handlers[Methods::RESOURCES_UNSUBSCRIBE] = block
end

#roots_list_changed_handler {|params| ... } ⇒ Object

Sets a handler for ‘notifications/roots/list_changed` notifications. Called when a client notifies the server that its filesystem roots have changed.

Yields:

  • (params)

    The notification params (typically ‘nil`).



275
276
277
# File 'lib/mcp/server.rb', line 275

def roots_list_changed_handler(&block)
  @handlers[Methods::NOTIFICATIONS_ROOTS_LIST_CHANGED] = block
end