Class: MCP::Server

Inherits:
Object
  • Object
show all
Includes:
Instrumentation, Pagination
Defined in:
lib/mcp/server.rb,
lib/mcp/server/pagination.rb,
lib/mcp/server/transports.rb,
lib/mcp/server/capabilities.rb,
lib/mcp/server/transports/stdio_transport.rb,
lib/mcp/server/transports/streamable_http_transport.rb

Defined Under Namespace

Modules: Pagination, Transports Classes: Capabilities, MethodAlreadyDefinedError, RequestHandlerError, URLElicitationRequiredError

Constant Summary collapse

DEFAULT_VERSION =
"0.1.0"
UNSUPPORTED_PROPERTIES_UNTIL_2025_06_18 =
[:description, :icons].freeze
UNSUPPORTED_PROPERTIES_UNTIL_2025_03_26 =
[:title, :websiteUrl].freeze
DEFAULT_COMPLETION_RESULT =
{ completion: { values: [], hasMore: false } }.freeze
MAX_COMPLETION_VALUES =

Servers return an array of completion values ranked by relevance, with maximum 100 items per response. modelcontextprotocol.io/specification/2025-11-25/server/utilities/completion#completion-results

100

Instance Attribute Summary collapse

Instance Method Summary collapse

Methods included from Instrumentation

#add_instrumentation_data, #instrument_call

Constructor Details

#initialize(description: nil, icons: [], name: "model_context_protocol", title: nil, version: DEFAULT_VERSION, website_url: nil, instructions: nil, tools: [], prompts: [], resources: [], resource_templates: [], server_context: nil, configuration: nil, capabilities: nil, page_size: nil, transport: nil) ⇒ Server

Returns a new instance of Server.



76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
135
136
137
# File 'lib/mcp/server.rb', line 76

def initialize(
  description: nil,
  icons: [],
  name: "model_context_protocol",
  title: nil,
  version: DEFAULT_VERSION,
  website_url: nil,
  instructions: nil,
  tools: [],
  prompts: [],
  resources: [],
  resource_templates: [],
  server_context: nil,
  configuration: nil,
  capabilities: nil,
  page_size: nil,
  transport: nil
)
  @description = description
  @icons = icons
  @name = name
  @title = title
  @version = version
  @website_url = website_url
  @instructions = instructions
  @tool_names = tools.map(&:name_value)
  @tools = tools.to_h { |t| [t.name_value, t] }
  @prompts = prompts.to_h { |p| [p.name_value, p] }
  @resources = resources
  @resource_templates = resource_templates
  @resource_index = index_resources_by_uri(resources)
  @server_context = server_context
  self.page_size = page_size
  @configuration = MCP.configuration.merge(configuration)
  @client = nil

  validate!

  @capabilities = capabilities || default_capabilities
  @client_capabilities = nil
  @logging_message_notification = nil

  @handlers = {
    Methods::RESOURCES_LIST => method(:list_resources),
    Methods::RESOURCES_READ => method(:read_resource_no_content),
    Methods::RESOURCES_TEMPLATES_LIST => method(:list_resource_templates),
    Methods::RESOURCES_SUBSCRIBE => ->(_) { {} },
    Methods::RESOURCES_UNSUBSCRIBE => ->(_) { {} },
    Methods::TOOLS_LIST => method(:list_tools),
    Methods::TOOLS_CALL => method(:call_tool),
    Methods::PROMPTS_LIST => method(:list_prompts),
    Methods::PROMPTS_GET => method(:get_prompt),
    Methods::INITIALIZE => method(:init),
    Methods::PING => ->(_) { {} },
    Methods::NOTIFICATIONS_INITIALIZED => ->(_) {},
    Methods::NOTIFICATIONS_PROGRESS => ->(_) {},
    Methods::NOTIFICATIONS_ROOTS_LIST_CHANGED => ->(_) {},
    Methods::COMPLETION_COMPLETE => ->(_) { DEFAULT_COMPLETION_RESULT },
    Methods::LOGGING_SET_LEVEL => method(:configure_logging_level),
  }
  @transport = transport
end

Instance Attribute Details

#capabilitiesObject

Returns the value of attribute capabilities.



73
74
75
# File 'lib/mcp/server.rb', line 73

def capabilities
  @capabilities
end

#client_capabilitiesObject (readonly)

Returns the value of attribute client_capabilities.



74
75
76
# File 'lib/mcp/server.rb', line 74

def client_capabilities
  @client_capabilities
end

#configurationObject

Returns the value of attribute configuration.



73
74
75
# File 'lib/mcp/server.rb', line 73

def configuration
  @configuration
end

#descriptionObject

Returns the value of attribute description.



73
74
75
# File 'lib/mcp/server.rb', line 73

def description
  @description
end

#iconsObject

Returns the value of attribute icons.



73
74
75
# File 'lib/mcp/server.rb', line 73

def icons
  @icons
end

#instructionsObject

Returns the value of attribute instructions.



73
74
75
# File 'lib/mcp/server.rb', line 73

def instructions
  @instructions
end

#logging_message_notificationObject

Returns the value of attribute logging_message_notification.



73
74
75
# File 'lib/mcp/server.rb', line 73

def logging_message_notification
  @logging_message_notification
end

#nameObject

Returns the value of attribute name.



73
74
75
# File 'lib/mcp/server.rb', line 73

def name
  @name
end

#page_sizeObject

Returns the value of attribute page_size.



74
75
76
# File 'lib/mcp/server.rb', line 74

def page_size
  @page_size
end

#promptsObject

Returns the value of attribute prompts.



73
74
75
# File 'lib/mcp/server.rb', line 73

def prompts
  @prompts
end

#resourcesObject

Returns the value of attribute resources.



73
74
75
# File 'lib/mcp/server.rb', line 73

def resources
  @resources
end

#server_contextObject

Returns the value of attribute server_context.



73
74
75
# File 'lib/mcp/server.rb', line 73

def server_context
  @server_context
end

#titleObject

Returns the value of attribute title.



73
74
75
# File 'lib/mcp/server.rb', line 73

def title
  @title
end

#toolsObject

Returns the value of attribute tools.



73
74
75
# File 'lib/mcp/server.rb', line 73

def tools
  @tools
end

#transportObject

Returns the value of attribute transport.



73
74
75
# File 'lib/mcp/server.rb', line 73

def transport
  @transport
end

#versionObject

Returns the value of attribute version.



73
74
75
# File 'lib/mcp/server.rb', line 73

def version
  @version
end

#website_urlObject

Returns the value of attribute website_url.



73
74
75
# File 'lib/mcp/server.rb', line 73

def website_url
  @website_url
end

Instance Method Details

#build_sampling_params(capabilities, messages:, max_tokens:, system_prompt: nil, model_preferences: nil, include_context: nil, temperature: nil, stop_sequences: nil, metadata: nil, tools: nil, tool_choice: nil) ⇒ Object



279
280
281
282
283
284
285
286
287
288
289
290
291
292
293
294
295
296
297
298
299
300
301
302
303
304
305
306
307
308
309
310
311
312
313
314
315
316
# File 'lib/mcp/server.rb', line 279

def build_sampling_params(
  capabilities,
  messages:,
  max_tokens:,
  system_prompt: nil,
  model_preferences: nil,
  include_context: nil,
  temperature: nil,
  stop_sequences: nil,
  metadata: nil,
  tools: nil,
  tool_choice: nil
)
  unless capabilities&.dig(:sampling)
    raise "Client does not support sampling."
  end

  if tools && !capabilities.dig(:sampling, :tools)
    raise "Client does not support sampling with tools."
  end

  if tool_choice && !capabilities.dig(:sampling, :tools)
    raise "Client does not support sampling with tool_choice."
  end

  {
    messages: messages,
    maxTokens: max_tokens,
    systemPrompt: system_prompt,
    modelPreferences: model_preferences,
    includeContext: include_context,
    temperature: temperature,
    stopSequences: stop_sequences,
    metadata: ,
    tools: tools,
    toolChoice: tool_choice,
  }.compact
end

#completion_handler {|params| ... } ⇒ Object

Sets a custom handler for ‘completion/complete` requests. The block receives the parsed request params and should return completion values.

Yields:

  • (params)

    The request params containing ‘:ref`, `:argument`, and optionally `:context`.

Yield Returns:

  • (Hash)

    A hash with ‘:completion` key containing `:values`, optional `:total`, and `:hasMore`.



257
258
259
# File 'lib/mcp/server.rb', line 257

def completion_handler(&block)
  @handlers[Methods::COMPLETION_COMPLETE] = block
end

#define_custom_method(method_name:, &block) ⇒ Object



182
183
184
185
186
187
188
# File 'lib/mcp/server.rb', line 182

def define_custom_method(method_name:, &block)
  if @handlers.key?(method_name)
    raise MethodAlreadyDefinedError, method_name
  end

  @handlers[method_name] = block
end

#define_prompt(name: nil, title: nil, description: nil, arguments: [], &block) ⇒ Object



175
176
177
178
179
180
# File 'lib/mcp/server.rb', line 175

def define_prompt(name: nil, title: nil, description: nil, arguments: [], &block)
  prompt = Prompt.define(name: name, title: title, description: description, arguments: arguments, &block)
  @prompts[prompt.name_value] = prompt

  validate!
end

#define_tool(name: nil, title: nil, description: nil, input_schema: nil, annotations: nil, meta: nil, &block) ⇒ Object



165
166
167
168
169
170
171
172
173
# File 'lib/mcp/server.rb', line 165

def define_tool(name: nil, title: nil, description: nil, input_schema: nil, annotations: nil, meta: nil, &block)
  tool = Tool.define(name: name, title: title, description: description, input_schema: input_schema, annotations: annotations, meta: meta, &block)
  tool_name = tool.name_value

  @tool_names << tool_name
  @tools[tool_name] = tool

  validate!
end

#handle(request, session: nil) ⇒ Hash?

Processes a parsed JSON-RPC request and returns the response as a Hash.

Parameters:

  • request (Hash)

    A parsed JSON-RPC request.

  • session (ServerSession, nil) (defaults to: nil)

    Per-connection session. Passed by ‘ServerSession#handle` for session-scoped notification delivery. When `nil`, progress and logging notifications from tool handlers are silently skipped.

Returns:

  • (Hash, nil)

    The JSON-RPC response, or ‘nil` for notifications.



146
147
148
149
150
# File 'lib/mcp/server.rb', line 146

def handle(request, session: nil)
  JsonRpcHandler.handle(request) do |method, request_id|
    handle_request(request, method, session: session, related_request_id: request_id)
  end
end

#handle_json(request, session: nil) ⇒ String?

Processes a JSON-RPC request string and returns the response as a JSON string.

Parameters:

  • request (String)

    A JSON-RPC request as a JSON string.

  • session (ServerSession, nil) (defaults to: nil)

    Per-connection session. Passed by ‘ServerSession#handle_json` for session-scoped notification delivery. When `nil`, progress and logging notifications from tool handlers are silently skipped.

Returns:

  • (String, nil)

    The JSON-RPC response as JSON, or ‘nil` for notifications.



159
160
161
162
163
# File 'lib/mcp/server.rb', line 159

def handle_json(request, session: nil)
  JsonRpcHandler.handle_json(request) do |method, request_id|
    handle_request(request, method, session: session, related_request_id: request_id)
  end
end

#notify_log_message(data:, level:, logger: nil) ⇒ Object



222
223
224
225
226
227
228
229
230
231
232
# File 'lib/mcp/server.rb', line 222

def notify_log_message(data:, level:, logger: nil)
  return unless @transport
  return unless logging_message_notification&.should_notify?(level)

  params = { "data" => data, "level" => level }
  params["logger"] = logger if logger

  @transport.send_notification(Methods::NOTIFICATIONS_MESSAGE, params)
rescue => e
  report_exception(e, { notification: "log_message" })
end

#notify_prompts_list_changedObject



206
207
208
209
210
211
212
# File 'lib/mcp/server.rb', line 206

def notify_prompts_list_changed
  return unless @transport

  @transport.send_notification(Methods::NOTIFICATIONS_PROMPTS_LIST_CHANGED)
rescue => e
  report_exception(e, { notification: "prompts_list_changed" })
end

#notify_resources_list_changedObject



214
215
216
217
218
219
220
# File 'lib/mcp/server.rb', line 214

def notify_resources_list_changed
  return unless @transport

  @transport.send_notification(Methods::NOTIFICATIONS_RESOURCES_LIST_CHANGED)
rescue => e
  report_exception(e, { notification: "resources_list_changed" })
end

#notify_tools_list_changedObject



198
199
200
201
202
203
204
# File 'lib/mcp/server.rb', line 198

def notify_tools_list_changed
  return unless @transport

  @transport.send_notification(Methods::NOTIFICATIONS_TOOLS_LIST_CHANGED)
rescue => e
  report_exception(e, { notification: "tools_list_changed" })
end

#resources_read_handler {|params| ... } ⇒ Object

Sets a custom handler for ‘resources/read` requests. The block receives the parsed request params and should return resource contents. The return value is set as the `contents` field of the response.

Yields:

  • (params)

    The request params containing ‘:uri`.

Yield Returns:

  • (Array<Hash>, Hash)

    Resource contents.



248
249
250
# File 'lib/mcp/server.rb', line 248

def resources_read_handler(&block)
  @handlers[Methods::RESOURCES_READ] = block
end

#resources_subscribe_handler {|params| ... } ⇒ Object

Sets a custom handler for ‘resources/subscribe` requests. The block receives the parsed request params. The return value is ignored; the response is always an empty result `{}` per the MCP specification.

Yields:

  • (params)

    The request params containing ‘:uri`.



266
267
268
# File 'lib/mcp/server.rb', line 266

def resources_subscribe_handler(&block)
  @handlers[Methods::RESOURCES_SUBSCRIBE] = block
end

#resources_unsubscribe_handler {|params| ... } ⇒ Object

Sets a custom handler for ‘resources/unsubscribe` requests. The block receives the parsed request params. The return value is ignored; the response is always an empty result `{}` per the MCP specification.

Yields:

  • (params)

    The request params containing ‘:uri`.



275
276
277
# File 'lib/mcp/server.rb', line 275

def resources_unsubscribe_handler(&block)
  @handlers[Methods::RESOURCES_UNSUBSCRIBE] = block
end

#roots_list_changed_handler {|params| ... } ⇒ Object

Sets a handler for ‘notifications/roots/list_changed` notifications. Called when a client notifies the server that its filesystem roots have changed.

Yields:

  • (params)

    The notification params (typically ‘nil`).



238
239
240
# File 'lib/mcp/server.rb', line 238

def roots_list_changed_handler(&block)
  @handlers[Methods::NOTIFICATIONS_ROOTS_LIST_CHANGED] = block
end